@@ -1,9 +1,6 @@
-import os
+# encoding: utf-8
 
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import os
 
 try:
     import watchdog
@@ -12,7 +9,8 @@ try:
 except ImportError:
     has_watchdog = False
 
-from bpython.test import mock
+from bpython.test import mock, unittest
+
 
 @unittest.skipUnless(has_watchdog, "watchdog required")
 class TestModuleChangeEventHandler(unittest.TestCase):
