@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 # The MIT License
 #
 # Copyright (c) 2015 Sebastian Ramacher
@@ -20,14 +22,16 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+from __future__ import absolute_import
+
 import re
 
 
 class LazyReCompile(object):
     """Compile regular expressions on first use
 
-    This class allows to store regular expressions and compiles them on first
-    use."""
+    This class allows one to store regular expressions and compiles them on
+    first use."""
 
     def __init__(self, regex, flags=0):
         self.regex = regex
