@@ -24,7 +24,7 @@
 
 # Modified by Brandon Navra
 # Notes for Windows
-# Prerequsites
+# Prerequisites
 #  - Curses
 #  - pyreadline
 #
@@ -39,7 +39,7 @@
 # - Instead the suspend key exits the program
 # - View source doesn't work on windows unless you install the less program (From GnuUtils or Cygwin)
 
-from __future__ import division, with_statement
+from __future__ import division, absolute_import
 
 import platform
 import os
@@ -52,39 +52,39 @@ import functools
 
 import struct
 if platform.system() != 'Windows':
-    import signal      #Windows does not have job control
-    import termios     #Windows uses curses
-    import fcntl       #Windows uses curses
+    import signal  # Windows does not have job control
+    import termios  # Windows uses curses
+    import fcntl  # Windows uses curses
 import unicodedata
 import errno
 
-from types import ModuleType
 from six.moves import range
 
 # These are used for syntax highlighting
 from pygments import format
 from pygments.formatters import TerminalFormatter
-from bpython._py3compat import PythonLexer
+from ._py3compat import PythonLexer
 from pygments.token import Token
-from bpython.formatter import BPythonFormatter
+from .formatter import BPythonFormatter
 
 # This for completion
-from bpython import importcompletion
+from . import importcompletion
 
 # This for config
-from bpython.config import Struct, getpreferredencoding
+from .config import Struct, getpreferredencoding
 
 # This for keys
-from bpython.keys import cli_key_dispatch as key_dispatch
+from .keys import cli_key_dispatch as key_dispatch
 
 # This for i18n
-from bpython import translations
-from bpython.translations import _
+from . import translations
+from .translations import _
 
-from bpython import repl
-from bpython._py3compat import py3
-from bpython.pager import page
-import bpython.args
+from . import repl
+from . import args as bpargs
+from ._py3compat import py3
+from .pager import page
+from .args import parse as argsparse
 
 if not py3:
     import inspect
@@ -113,6 +113,7 @@ def calculate_screen_lines(tokens, width, cursor=0):
             pos %= width
     return lines
 
+
 def forward_if_not_current(func):
     @functools.wraps(func)
     def newfunc(self, *args, **kwargs):
@@ -203,7 +204,7 @@ class FakeStdin(object):
                     # C-d
                     return ''
                 elif (key not in ('\n', '\r') and
-                    (len(key) > 1 or unicodedata.category(key) == 'Cc')):
+                      (len(key) > 1 or unicodedata.category(key) == 'Cc')):
                     continue
                 sys.stdout.write(key)
                 # Include the \n in the buffer - raw_input() seems to deal with trailing
@@ -281,8 +282,8 @@ def make_colors(config):
     }
 
     if platform.system() == 'Windows':
-        c = dict(c.items() +
-            [
+        c = dict(list(c.items()) +
+                 [
             ('K', 8),
             ('R', 9),
             ('G', 10),
@@ -291,8 +292,8 @@ def make_colors(config):
             ('M', 13),
             ('C', 14),
             ('W', 15),
-            ]
-         )
+        ]
+        )
 
     for i in range(63):
         if i > 7:
@@ -317,7 +318,6 @@ class CLIInteraction(repl.Interaction):
 
         return reply.lower() in (_('y'), _('yes'))
 
-
     def notify(self, s, n=10, wait_for_keypress=False):
         return self.statusbar.message(s, n)
 
@@ -331,7 +331,7 @@ class CLIRepl(repl.Repl):
         repl.Repl.__init__(self, interp, config)
         self.interp.writetb = self.writetb
         self.scr = scr
-        self.stdout_hist = ''
+        self.stdout_hist = ''  # native str (bytes in Py2, unicode in Py3)
         self.list_win = newwin(get_colpair(config, 'background'), 1, 1, 1, 1)
         self.cpos = 0
         self.do_exit = False
@@ -351,6 +351,7 @@ class CLIRepl(repl.Repl):
 
     def _get_cursor_offset(self):
         return len(self.s) - self.cpos
+
     def _set_cursor_offset(self, offset):
         self.cpos = len(self.s) - offset
     cursor_offset = property(_get_cursor_offset, _set_cursor_offset, None,
@@ -428,13 +429,13 @@ class CLIRepl(repl.Repl):
         it and force syntax highlighting."""
 
         if (self.paste_mode
-            and time.time() - self.last_key_press > self.config.paste_time):
+                and time.time() - self.last_key_press > self.config.paste_time):
             self.paste_mode = False
             self.print_line(self.s)
 
     def clear_current_line(self):
         """Called when a SyntaxError occurred in the interpreter. It is
-        used to prevent autoindentation from occuring after a
+        used to prevent autoindentation from occurring after a
         traceback."""
         repl.Repl.clear_current_line(self)
         self.s = ''
@@ -449,19 +450,21 @@ class CLIRepl(repl.Repl):
             self.scr.clrtoeol()
 
     def complete(self, tab=False):
-        """Get Autcomplete list and window.
+        """Get Autocomplete list and window.
 
         Called whenever these should be updated, and called
         with tab
         """
         if self.paste_mode:
-            self.scr.touchwin() #TODO necessary?
+            self.scr.touchwin()  # TODO necessary?
             return
 
         list_win_visible = repl.Repl.complete(self, tab)
         if list_win_visible:
             try:
-                self.show_list(self.matches_iter.matches, topline=self.argspec, formatter=self.matches_iter.completer.format)
+                self.show_list(self.matches_iter.matches, self.arg_pos,
+                               topline=self.funcprops,
+                               formatter=self.matches_iter.completer.format)
             except curses.error:
                 # XXX: This is a massive hack, it will go away when I get
                 # cusswords into a good enough state that we can start
@@ -488,6 +491,7 @@ class CLIRepl(repl.Repl):
 
     def _get_current_line(self):
         return self.s
+
     def _set_current_line(self, line):
         self.s = line
     current_line = property(_get_current_line, _set_current_line, None,
@@ -691,7 +695,7 @@ class CLIRepl(repl.Repl):
         self.print_line(self.s, newline=True)
         self.echo("\n")
 
-    def mkargspec(self, topline, down):
+    def mkargspec(self, topline, in_arg, down):
         """This figures out what to do with the argspec and puts it nicely into
         the list window. It returns the number of lines used to display the
         argspec.  It's also kind of messy due to it having to call so many
@@ -699,16 +703,15 @@ class CLIRepl(repl.Repl):
         sturdy."""
 
         r = 3
-        fn = topline[0]
-        args = topline[1][0]
-        kwargs = topline[1][3]
-        _args = topline[1][1]
-        _kwargs = topline[1][2]
-        is_bound_method = topline[2]
-        in_arg = topline[3]
+        fn = topline.func
+        args = topline.argspec.args
+        kwargs = topline.argspec.defaults
+        _args = topline.argspec.varargs
+        _kwargs = topline.argspec.varkwargs
+        is_bound_method = topline.is_bound_method
         if py3:
-            kwonly = topline[1][4]
-            kwonly_defaults = topline[1][5] or dict()
+            kwonly = topline.argspec.kwonly
+            kwonly_defaults = topline.argspec.kwonly_defaults or dict()
         max_w = int(self.scr.getmaxyx()[1] * 0.6)
         self.list_win.erase()
         self.list_win.resize(3, max_w)
@@ -716,7 +719,7 @@ class CLIRepl(repl.Repl):
 
         self.list_win.addstr('\n  ')
         self.list_win.addstr(fn,
-            get_colpair(self.config, 'name') | curses.A_BOLD)
+                             get_colpair(self.config, 'name') | curses.A_BOLD)
         self.list_win.addstr(': (', get_colpair(self.config, 'name'))
         maxh = self.scr.getmaxyx()[0]
 
@@ -765,7 +768,7 @@ class CLIRepl(repl.Repl):
             if kw is not None:
                 self.list_win.addstr('=', punctuation_colpair)
                 self.list_win.addstr(kw, get_colpair(self.config, 'token'))
-            if k != len(args) -1:
+            if k != len(args) - 1:
                 self.list_win.addstr(', ', punctuation_colpair)
 
         if _args:
@@ -854,7 +857,7 @@ class CLIRepl(repl.Repl):
             key = '\n'
             # Don't return; let it get handled
 
-        if key == chr(27): #Escape Key
+        if key == chr(27):  # Escape Key
             return ''
 
         if key in (BACKSP, 'KEY_BACKSPACE'):
@@ -894,7 +897,7 @@ class CLIRepl(repl.Repl):
             self.fwd()
             return ''
 
-        elif key in ("KEY_LEFT",' ^B', chr(2)):  # Cursor Left or ^B
+        elif key in ("KEY_LEFT", ' ^B', chr(2)):  # Cursor Left or ^B
             self.mvc(1)
             # Redraw (as there might have been highlighted parens)
             self.print_line(self.s)
@@ -914,11 +917,11 @@ class CLIRepl(repl.Repl):
             # Redraw (as there might have been highlighted parens)
             self.print_line(self.s)
 
-        elif key in ("KEY_NPAGE", '\T'): # page_down or \T
+        elif key in ("KEY_NPAGE", '\T'):  # page_down or \T
             self.hend()
             self.print_line(self.s)
 
-        elif key in ("KEY_PPAGE", '\S'): # page_up or \S
+        elif key in ("KEY_PPAGE", '\S'):  # page_up or \S
             self.hbegin()
             self.print_line(self.s)
 
@@ -973,7 +976,7 @@ class CLIRepl(repl.Repl):
             try:
                 source = self.get_source_of_current_name()
             except repl.SourceNotFound as e:
-                self.statusbar.message(_(e))
+                self.statusbar.message('%s' % (e, ))
             else:
                 if config.highlight_show_source:
                     source = format(PythonLexer().get_tokens(source),
@@ -1065,15 +1068,23 @@ class CLIRepl(repl.Repl):
     def prompt(self, more):
         """Show the appropriate Python prompt"""
         if not more:
-            self.echo("\x01%s\x03%s" % (self.config.color_scheme['prompt'], self.ps1))
-            self.stdout_hist += self.ps1
+            self.echo("\x01%s\x03%s" %
+                      (self.config.color_scheme['prompt'], self.ps1))
+            if py3:
+                self.stdout_hist += self.ps1
+            else:
+                self.stdout_hist += self.ps1.encode(getpreferredencoding())
             self.s_hist.append('\x01%s\x03%s\x04' %
                                (self.config.color_scheme['prompt'], self.ps1))
         else:
             prompt_more_color = self.config.color_scheme['prompt_more']
             self.echo("\x01%s\x03%s" % (prompt_more_color, self.ps2))
-            self.stdout_hist += self.ps2
-            self.s_hist.append('\x01%s\x03%s\x04' % (prompt_more_color, self.ps2))
+            if py3:
+                self.stdout_hist += self.ps2
+            else:
+                self.stdout_hist += self.ps2.encode(getpreferredencoding())
+            self.s_hist.append('\x01%s\x03%s\x04' %
+                               (prompt_more_color, self.ps2))
 
     def push(self, s, insert_into_history=True):
         # curses.raw(True) prevents C-c from causing a SIGINT
@@ -1097,7 +1108,7 @@ class CLIRepl(repl.Repl):
             self.iy, self.ix = self.scr.getyx()
             for i in s.split('\x04'):
                 self.echo(i, redraw=False)
-            if k < len(self.s_hist) -1:
+            if k < len(self.s_hist) - 1:
                 self.scr.addstr('\n')
         self.iy, self.ix = self.scr.getyx()
         self.print_line(self.s)
@@ -1180,14 +1191,12 @@ class CLIRepl(repl.Repl):
         self.statusbar.resize(refresh=False)
         self.redraw()
 
-
     def getstdout(self):
         """This method returns the 'spoofed' stdout buffer, for writing to a
         file or sending to a pastebin or whatever."""
 
         return self.stdout_hist + '\n'
 
-
     def reevaluate(self):
         """Clear the buffer, redraw the screen and re-evaluate the history"""
 
@@ -1228,7 +1237,7 @@ class CLIRepl(repl.Repl):
 
         self.evaluating = False
         #map(self.push, self.history)
-        #^-- That's how simple this method was at first :(
+        # ^-- That's how simple this method was at first :(
 
     def write(self, s):
         """For overriding stdout defaults"""
@@ -1252,8 +1261,7 @@ class CLIRepl(repl.Repl):
         self.echo(s)
         self.s_hist.append(s.rstrip())
 
-
-    def show_list(self, items, topline=None, formatter=None, current_item=None):
+    def show_list(self, items, arg_pos, topline=None, formatter=None, current_item=None):
 
         shared = Struct()
         shared.cols = 0
@@ -1275,7 +1283,7 @@ class CLIRepl(repl.Repl):
                 current_item = formatter(current_item)
 
         if topline:
-            height_offset = self.mkargspec(topline, down) + 1
+            height_offset = self.mkargspec(topline, arg_pos, down) + 1
         else:
             height_offset = 0
 
@@ -1339,7 +1347,7 @@ class CLIRepl(repl.Repl):
             self.list_win.resize(rows + 2, w)
         else:
             docstring = self.format_docstring(self.docstring, max_w - 2,
-                max_h - height_offset)
+                                              max_h - height_offset)
             docstring_string = ''.join(docstring)
             rows += len(docstring)
             self.list_win.resize(rows, max_w)
@@ -1374,7 +1382,7 @@ class CLIRepl(repl.Repl):
                                  get_colpair(self.config, 'comment'))
             # XXX: After all the trouble I had with sizing the list box (I'm not very good
             # at that type of thing) I decided to do this bit of tidying up here just to
-            # make sure there's no unnececessary blank lines, it makes things look nicer.
+            # make sure there's no unnecessary blank lines, it makes things look nicer.
 
         y = self.list_win.getyx()[0]
         self.list_win.resize(y + 2, w)
@@ -1395,7 +1403,7 @@ class CLIRepl(repl.Repl):
 
     def size(self):
         """Set instance attributes for x and y top left corner coordinates
-        and width and heigth for the window."""
+        and width and height for the window."""
         global stdscr
         h, w = stdscr.getmaxyx()
         self.y = 0
@@ -1439,7 +1447,7 @@ class CLIRepl(repl.Repl):
 
         # 3. check to see if we can expand the current word
         if self.matches_iter.is_cseq():
-            #TODO resolve this error-prone situation:
+            # TODO resolve this error-prone situation:
             # can't assign at same time to self.s and self.cursor_offset
             # because for cursor_offset
             # property to work correctly, self.s must already be set
@@ -1452,9 +1460,10 @@ class CLIRepl(repl.Repl):
         # 4. swap current word for a match list item
         elif self.matches_iter.matches:
             current_match = back and self.matches_iter.previous() \
-                                  or next(self.matches_iter)
+                or next(self.matches_iter)
             try:
-                self.show_list(self.matches_iter.matches, topline=self.argspec,
+                self.show_list(self.matches_iter.matches, self.arg_pos,
+                               topline=self.funcprops,
                                formatter=self.matches_iter.completer.format,
                                current_item=current_match)
             except curses.error:
@@ -1493,7 +1502,7 @@ class CLIRepl(repl.Repl):
             return ''
 
         self.f_string = ''
-        self.cpos = -1 # Set cursor position to -1 to prevent paren matching
+        self.cpos = -1  # Set cursor position to -1 to prevent paren matching
 
         self.iy, self.ix = self.scr.getyx()
         self.evaluating = True
@@ -1524,6 +1533,7 @@ class CLIRepl(repl.Repl):
         self.scr.redrawwin()
         return ''
 
+
 class Statusbar(object):
     """This class provides the status bar at the bottom of the screen.
     It has message() and prompt() methods for user interactivity, as
@@ -1564,7 +1574,7 @@ class Statusbar(object):
 
     def size(self):
         """Set instance attributes for x and y top left corner coordinates
-        and width and heigth for the window."""
+        and width and height for the window."""
         h, w = gethw()
         self.y = h - 1
         self.w = w
@@ -1680,7 +1690,7 @@ class Statusbar(object):
 def init_wins(scr, config):
     """Initialise the two windows (the main repl interface and the little
     status bar at the bottom with some stuff in it)"""
-    #TODO: Document better what stuff is on the status bar.
+    # TODO: Document better what stuff is on the status bar.
 
     background = get_colpair(config, 'background')
     h, w = gethw()
@@ -1712,11 +1722,13 @@ def sigwinch(unused_scr):
     global DO_RESIZE
     DO_RESIZE = True
 
+
 def sigcont(unused_scr):
     sigwinch(unused_scr)
     # Forces the redraw
     curses.ungetch('\x00')
 
+
 def gethw():
     """I found this code on a usenet post, and snipped out the bit I needed,
     so thanks to whoever wrote that, sorry I forgot your name, I'm sure you're
@@ -1754,7 +1766,8 @@ def gethw():
             sizex = right - left + 1
             sizey = bottom - top + 1
         else:
-            sizex, sizey = stdscr.getmaxyx()# can't determine actual size - return default values
+            # can't determine actual size - return default values
+            sizex, sizey = stdscr.getmaxyx()
 
         h, w = sizey, sizex
     return h, w
@@ -1789,7 +1802,7 @@ def do_resize(caller):
     global DO_RESIZE
     h, w = gethw()
     if not h:
-    # Hopefully this shouldn't happen. :)
+        # Hopefully this shouldn't happen. :)
         return
 
     curses.endwin()
@@ -1871,7 +1884,8 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
         old_sigwinch_handler = signal.signal(signal.SIGWINCH,
                                              lambda *_: sigwinch(scr))
         # redraw window after being suspended
-        old_sigcont_handler = signal.signal(signal.SIGCONT, lambda *_: sigcont(scr))
+        old_sigcont_handler = signal.signal(
+            signal.SIGCONT, lambda *_: sigcont(scr))
 
     stdscr = scr
     try:
@@ -1889,9 +1903,6 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
     curses.raw(True)
     main_win, statusbar = init_wins(scr, config)
 
-    if locals_ is None:
-        sys.modules['__main__'] = ModuleType('__main__')
-        locals_ = sys.modules['__main__'].__dict__
     interpreter = repl.Interpreter(locals_, getpreferredencoding())
 
     clirepl = CLIRepl(main_win, interpreter, statusbar, config, idle)
@@ -1904,7 +1915,7 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
     if args:
         exit_value = ()
         try:
-            bpython.args.exec_code(interpreter, args)
+            bpargs.exec_code(interpreter, args)
         except SystemExit as e:
             # The documentation of code.InteractiveInterpreter.runcode claims
             # that it reraises SystemExit. However, I can't manage to trigger
@@ -1947,8 +1958,7 @@ def main_curses(scr, args, config, interactive=True, locals_=None,
 def main(args=None, locals_=None, banner=None):
     translations.init()
 
-
-    config, options, exec_args = bpython.args.parse(args)
+    config, options, exec_args = argsparse(args)
 
     # Save stdin, stdout and stderr for later restoration
     orig_stdin = sys.stdin
@@ -1971,6 +1981,7 @@ def main(args=None, locals_=None, banner=None):
         sys.stdout.flush()
     return repl.extract_exit_value(exit_value)
 
+
 if __name__ == '__main__':
     sys.exit(main())
 
